#ifndef ENTITIES_H
#define ENTITIES_H

#include <cstdint>
#include <memory>
#include <string>

#include <odb/core.hxx>
#include <odb/lazy-ptr.hxx>


PRAGMA_DB(object)
class Child
{
public:

    Child(const std::string& field1, int32_t field2) :
        m_field1(field1),
        m_field2(field2)
    {
    }

private:

    friend class odb::access;

    Child()
    {
    }

    PRAGMA_DB(id auto)
    int64_t         m_id = 0;
    std::string     m_field1;
    int32_t         m_field2 = 0;
};

using ChildPtr = std::shared_ptr<Child>;



PRAGMA_DB(object)
class Parent
{
public:

    Parent(const std::string& field1, int32_t field2, ChildPtr childPtr) :
        m_field1(field1),
        m_field2(field2),
        m_child(childPtr)
    {
    }

    int64_t id() const
    {
        return m_id;
    }

    ChildPtr child() const
    {
        return m_child.load();
    }

private:

    friend class odb::access;

    Parent()
    {
    }


    PRAGMA_DB(id auto)
    int64_t                         m_id = 0;
    std::string                     m_field1;
    int32_t                         m_field2 = 0;
    odb::lazy_shared_ptr<Child>     m_child;
};

using ParentPtr = std::shared_ptr<Parent>;







#endif // ENTITIES_H
