// odb_weak_pointers.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <memory>
#include <string>

#include <boost/filesystem.hpp>

#include <odb/connection.hxx>
#include <odb/database.hxx>
#include <odb/schema-catalog.hxx>
#include <odb/transaction.hxx>
#include <odb/sqlite/database.hxx>

#include "entities.h"
#include "odb/entities-odb.hxx"



std::unique_ptr<odb::database> create_database()
{
    std::string dbPath = "db_test.db";

    boost::filesystem::remove(dbPath);

    std::unique_ptr<odb::database> db(new odb::sqlite::database(dbPath, SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE));

    {
        odb::connection_ptr c(db->connection());

        c->execute("PRAGMA foreign_keys=OFF");

        try
        {
            odb::transaction t(c->begin());
            odb::schema_catalog::create_schema(*db, std::string(), true);
            t.commit();
        }
        catch (const std::exception& e)
        {
            std::cerr << "create_database() - " << e.what() << std::endl;
        }
        catch (...)
        {
            std::cerr << "create_database() - unknown error" << std::endl;
        }

        c->execute("PRAGMA foreign_keys=ON");
    }

    return db;
}


int64_t test_store(std::unique_ptr<odb::database>& db)
{
    odb::transaction t(db->begin());

    ChildPtr child = std::make_shared<Child>("child", 123);
    ParentPtr parent = std::make_shared<Parent>("parent", 987, child);

    db->persist(child);
    db->persist(parent);

    t.commit();

    return parent->id();
}


void test_load(std::unique_ptr<odb::database>& db, int64_t id)
{
    odb::transaction t(db->begin());

    ParentPtr parent(db->load<Parent>(id));

    ChildPtr child = parent->child();

    t.commit();
}




int _tmain(int argc, _TCHAR* argv[])
{
    std::unique_ptr<odb::database> db = create_database();

    int64_t id = test_store(db);

    test_load(db, id);

	return 0;
}

